


#ifndef __CLOCK_TYPEBUILTINS_H__
#define __CLOCK_TYPEBUILTINS_H__ 1

#ifdef __cplusplus
extern "C" {
#endif


/* --- ../../applets/clock/clock-utils.h --- */
#define CLOCK_TYPE_FORMAT clock_format_get_type()
GType clock_format_get_type (void);
#ifdef __cplusplus
}
#endif

#endif /* __CLOCK_TYPEBUILTINS_H__ */



