#ifndef PANEL_ICON_NAMES_H
#define PANEL_ICON_NAMES_H

#define PANEL_ICON_BOOKMARKS		"user-bookmarks"
#define PANEL_ICON_BURNER		"caja-cd-burner"
#define PANEL_ICON_COMPUTER		"computer"
#define PANEL_ICON_DESKTOP		"user-desktop"
#define PANEL_ICON_DRAWER		"mate-panel-drawer"
#define PANEL_ICON_FILESYSTEM		"drive-harddisk"
#define PANEL_ICON_FOLDER		"folder"
#define PANEL_ICON_FOLDER_DOCUMENTS	"folder-documents"
#define PANEL_ICON_FOLDER_DOWNLOAD	"folder-download"
#define PANEL_ICON_FOLDER_MUSIC		"folder-music"
#define PANEL_ICON_FOLDER_PICTURES	"folder-pictures"
#define PANEL_ICON_FOLDER_PUBLIC_SHARE	"folder-publicshare"
#define PANEL_ICON_FOLDER_TEMPLATES	"folder-templates"
#define PANEL_ICON_FOLDER_VIDEOS	"folder-videos"
#define PANEL_ICON_FONTS		"preferences-desktop-font"
#define PANEL_ICON_FORCE_QUIT		"mate-panel-force-quit"
#define PANEL_ICON_HOME			"user-home"
#define PANEL_ICON_LAUNCHER		"mate-panel-launcher"
#define PANEL_ICON_LOCKSCREEN		"system-lock-screen"
#define PANEL_ICON_LOGOUT		"system-log-out"
#define PANEL_ICON_MAIN_MENU		"start-here"
#define PANEL_ICON_NETWORK		"network-workgroup"
#define PANEL_ICON_NETWORK_SERVER	"network-server"
#define PANEL_ICON_PANEL		"mate-panel"
#define PANEL_ICON_RECENT		"document-open-recent"
#define PANEL_ICON_REMOTE		"applications-internet"
#define PANEL_ICON_REMOVABLE_MEDIA	"drive-removable-media"
#define PANEL_ICON_RUN			"system-run"
#define PANEL_ICON_SEPARATOR		"mate-panel-separator"
#define PANEL_ICON_SAVED_SEARCH		"folder-saved-search"
#define PANEL_ICON_SEARCHTOOL		"system-search"
#define PANEL_ICON_SHUTDOWN		"system-shutdown"
#define PANEL_ICON_THEME		"preferences-desktop-theme"
#define PANEL_ICON_TRASH		"user-trash"
#define PANEL_ICON_UNKNOWN		"image-missing"

#endif /* PANEL_ICON_NAMES_H */
